# -*- coding:utf-8 -*-
"""
Author: fandi.di@bytedance.com
"""


# small is from big
def contains(big, small) -> bool:
    for i in small:
        if i not in big:
            return False
    return True


# ab不相交
def has_no_same(a, b) -> bool:
    for i in a:
        if i in b:
            return False
    return True


class Humanoise(object):
    """ Humans. Add your noise.
        params
            - train_path  = str or list. The file path of train data.
            - val_path    = str or list. The file path of val data.
            - sample_path = str or list. The file path of sample data.
            - cols        = list. ALL column names of data, must match data file.
            - cate_cols   = list. Column names, which you need for category features.
            - num_cols    = list. Column names, which you need for numeric features.
            - hash_cols   = list. Column names, which you need for hash features.
            - var_len_cols= list. Column names, which you need for var len features.
            - label_cols  = list. Column names, prediction target.
            - pre_emb_len_map = dict. Column names if pre-train with their pretrain emb length.
            - field_map   = dict. ALL category-like k-v map dict.
            - maxlen_map  = dict. The length of var_len_cols.
            - bucket_map  = dict. The bucket for some category features.
            - share_map   = dict. The columns which share embedding.
            - strict_line_default_type_dict = dict. The strict line default value.
    """

    def __init__(self, train_path='', val_path='', sample_path='', cols=[], cate_cols=[], num_cols=[], hash_cols=[],
                 pre_emb_len_map={}, var_len_cols=[], label_cols=[], need_encode_map={}, field_map={}, field_diy_map={},
                 maxlen_map={}, bucket_map={}, share_map={}, strict_line_default_type_dict={}):
        self.train_path = train_path
        self.val_path = val_path
        self.sample_path = sample_path

        self.cols = cols
        self.cate_cols = cate_cols
        self.num_cols = num_cols
        self.hash_cols = hash_cols
        self.var_len_cols = var_len_cols
        self.label_cols = label_cols

        self.pre_emb_len_map = pre_emb_len_map
        self.need_encode_map = need_encode_map
        self.field_map = field_map
        self.field_diy_map = field_diy_map
        for col in self.need_encode_map.keys():
            if col not in self.field_map:
                self.field_map[col] = list(range(2 * len(self.need_encode_map[col])))
        self.maxlen_map = maxlen_map
        self.bucket_map = bucket_map
        self.share_map = share_map
        self.strict_line_default_type_dict = strict_line_default_type_dict

        # all type columns are from cols
        assert contains(cols, cate_cols)
        assert contains(cols, num_cols)
        assert contains(cols, hash_cols)
        assert contains(cols, var_len_cols)
        assert contains(cols, label_cols)
        assert contains(cols, pre_emb_len_map)

        # all type columns has no same
        assert has_no_same(cate_cols, num_cols)
        assert has_no_same(cate_cols, hash_cols)
        assert has_no_same(cate_cols, var_len_cols)
        assert has_no_same(cate_cols, label_cols)
        assert has_no_same(num_cols, hash_cols)
        assert has_no_same(num_cols, var_len_cols)
        assert has_no_same(num_cols, label_cols)
        assert has_no_same(hash_cols, var_len_cols)
        assert has_no_same(hash_cols, label_cols)
        assert has_no_same(var_len_cols, label_cols)
        # cols and label_cols CANNOT be empty
        assert len(cols) * len(label_cols) > 0

        assert contains(cate_cols, bucket_map)
        assert contains(var_len_cols, maxlen_map)


